% Calculate multiple pore size distributions from PALS results stored in
% XLS files and analyze them
% 
% Copyright (C) 2016 Radek Zaleski e-mail: radek@zaleski.umcs.pl
%
% Requires: 
% tauD_cyl_T=295K_delta=0.166nm.mat - Extended Tao-Eldrup model data
% XAD4+C7_ads.xls, XAD4+C7_des.xls - PALS results

% PARAMETRS
Dcut=5.3; % nm

% CORE
% load results of the ETE model 
load('tauD_cyl_T=295K_delta=0.166nm.mat')

% identify small and large pores
idxs=find(D<Dcut);
idxl=find(D>=Dcut);

% data for PSD model
dtau=gradient(tau);
dtaudD=gradient(tau,D);
dtaudlogD=gradient(tau,log(D));

% load data
fname=['XAD4+C7_ads.xls'
       'XAD4+C7_des.xls'];

clear imp ps is ts ss I dVdD dVdlogD Dmax FWHM V Vs Vl
for ii=1:size(fname,1) % adsorption/desorption
    imp(ii)=importdata(fname(ii,:));
    ps{ii}=imp(ii).data(:,3); % pressure
    is{ii}=imp(ii).data(:,12); % intensity
    ts{ii}=imp(ii).data(:,21); % lifetime
    ss{ii}=imp(ii).data(:,23); % dispersion

    % remove result with not complete set of data (I, tau and sigma)
    idx=~((is{ii}==0)+(ts{ii}==0)+(ss{ii}==0)+(isnan(is{ii}))+(isnan(ts{ii}))+(isnan(ss{ii})));
    ps{ii}=ps{ii}(idx);
    is{ii}=is{ii}(idx);
    ts{ii}=ts{ii}(idx);
    ss{ii}=ss{ii}(idx);
    
    % parameters of lognormal distribution 
    sigma=sqrt(log((ss{ii}./ts{ii}).^2+1));        
    tau0=ts{ii}./exp(sigma.^2/2);
    % prepare matrices
    taus=repmat(tau',size(is{ii}));
    ints=repmat(is{ii},size(tau'));
    sigmas=repmat(sigma,size(tau'));
    tau0s=repmat(tau0,size(tau'));
    % lifetime distribution
    I{ii}=ints./(sqrt(2*pi)*sigmas).*exp(-((log(tau0s./taus)).^2)./(2*sigmas.^2))./taus;
    
    % PSD
    dVdD{ii}=I{ii}.*repmat(dtaudD',size(is{ii}));
    dVdlogD{ii}=I{ii}.*repmat(dtaudlogD',size(is{ii}));

    % maxima
    [dVdDmax,ind]=max(dVdD{ii},[],2);
    Dmax{ii}=D(ind);
    
    % FWHM
    Dabv=repmat(D,size(is{ii}')).*(dVdD{ii}>repmat(dVdDmax/2,size(tau')))';
    Dabv(Dabv==0)=NaN;
    FWHM{ii}=(max(Dabv)-min(Dabv))';
    
    % volume
    V{ii}=trapz(D,dVdD{ii},2);
    Vmax=max(V{ii});
    V{ii}=V{ii}/Vmax*100;
    Vs{ii}=trapz(D(idxs),dVdD{ii}(:,idxs),2)/Vmax*100;
    Vl{ii}=trapz(D(idxl),dVdD{ii}(:,idxl),2)/Vmax*100;
end

% FIGURES
figure
subplot(1,2,1)
semilogx(D(1<D&D<100),dVdlogD{1}(:,1<D&D<100))
ylabel('dVdlog(D) (nm^{-1})');
xlabel('D (nm)');
title('PSD adsorption');
subplot(1,2,2)
semilogx(D(1<D&D<100),dVdlogD{2}(:,1<D&D<100))
ylabel('dVdlog(D) (nm^{-1})');
xlabel('D (nm)');
title('PSD desorption');

figure
subplot(1,2,1)
plot(ps{1},Dmax{1},'ob',ps{2},Dmax{2},'or')
ylabel('D_{max} (nm)')
xlabel('p/p_0')
title('Pore size at maximum');
subplot(1,2,2)
plot(ps{1},FWHM{1},'ob',ps{2},FWHM{2},'or')
ylabel('FWHM (nm)')
xlabel('p/p_0')
title('PSD width at half maximum');

figure
subplot(2,2,[1,3])
plot(ps{1},V{1},'ob',ps{2},V{2},'or')
ylabel('V (%)')
xlabel('p/p_0')
title('Total volume');
subplot(2,2,2)
plot(ps{1},Vs{1},'ob',ps{2},Vs{2},'or')
ylabel('V_s (%)')
xlabel('p/p_0')
title('Volume of small pores');
subplot(2,2,4)
plot(ps{1},Vl{1},'ob',ps{2},Vl{2},'or')
ylabel('V_l (%)')
xlabel('p/p_0')
title('Volume of large pores');

% RESULTS
% write data to files
xlswrite('XAD4+C7H16_ads_dVdD.xls',...
    [[cellstr('D [nm]'),cellstr(num2str(ps{1},'p=%g'))'];...
    [num2cell(D),num2cell(dVdD{1}')]])
xlswrite('XAD4+C7H16_des_dVdD.xls',...
    [[cellstr('D [nm]'),cellstr(num2str(ps{2},'p=%g'))'];...
    [num2cell(D),num2cell(dVdD{2}')]])
xlswrite('XAD4+C7H16_ads_params.xls',...
    [[cellstr('p [MPa]'),cellstr('Dmax [nm]'),cellstr('FWHM [nm]'),...
    cellstr('V'),cellstr('Vs'),cellstr('Vl')];...
    [num2cell(ps{1}),num2cell([Dmax{1},FWHM{1},V{1},Vs{1},Vl{1}])]])
xlswrite('XAD4+C7H16_des_params.xls',...
    [[cellstr('p [MPa]'),cellstr('Dmax [nm]'),cellstr('FWHM [nm]'),...
    cellstr('V'),cellstr('Vs'),cellstr('Vl')];...
    [num2cell(ps{2}),num2cell([Dmax{2},FWHM{2},V{2},Vs{2},Vl{2}])]])